/*
Copyright 2022 Rancher Labs, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v3

import (
	"context"
	"time"

	"github.com/rancher/lasso/pkg/client"
	"github.com/rancher/lasso/pkg/controller"
	v3 "github.com/rancher/rancher/pkg/apis/management.cattle.io/v3"
	"github.com/rancher/wrangler/pkg/generic"
	"k8s.io/apimachinery/pkg/api/equality"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type RancherUserNotificationHandler func(string, *v3.RancherUserNotification) (*v3.RancherUserNotification, error)

type RancherUserNotificationController interface {
	generic.ControllerMeta
	RancherUserNotificationClient

	OnChange(ctx context.Context, name string, sync RancherUserNotificationHandler)
	OnRemove(ctx context.Context, name string, sync RancherUserNotificationHandler)
	Enqueue(name string)
	EnqueueAfter(name string, duration time.Duration)

	Cache() RancherUserNotificationCache
}

type RancherUserNotificationClient interface {
	Create(*v3.RancherUserNotification) (*v3.RancherUserNotification, error)
	Update(*v3.RancherUserNotification) (*v3.RancherUserNotification, error)

	Delete(name string, options *metav1.DeleteOptions) error
	Get(name string, options metav1.GetOptions) (*v3.RancherUserNotification, error)
	List(opts metav1.ListOptions) (*v3.RancherUserNotificationList, error)
	Watch(opts metav1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v3.RancherUserNotification, err error)
}

type RancherUserNotificationCache interface {
	Get(name string) (*v3.RancherUserNotification, error)
	List(selector labels.Selector) ([]*v3.RancherUserNotification, error)

	AddIndexer(indexName string, indexer RancherUserNotificationIndexer)
	GetByIndex(indexName, key string) ([]*v3.RancherUserNotification, error)
}

type RancherUserNotificationIndexer func(obj *v3.RancherUserNotification) ([]string, error)

type rancherUserNotificationController struct {
	controller    controller.SharedController
	client        *client.Client
	gvk           schema.GroupVersionKind
	groupResource schema.GroupResource
}

func NewRancherUserNotificationController(gvk schema.GroupVersionKind, resource string, namespaced bool, controller controller.SharedControllerFactory) RancherUserNotificationController {
	c := controller.ForResourceKind(gvk.GroupVersion().WithResource(resource), gvk.Kind, namespaced)
	return &rancherUserNotificationController{
		controller: c,
		client:     c.Client(),
		gvk:        gvk,
		groupResource: schema.GroupResource{
			Group:    gvk.Group,
			Resource: resource,
		},
	}
}

func FromRancherUserNotificationHandlerToHandler(sync RancherUserNotificationHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v3.RancherUserNotification
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v3.RancherUserNotification))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *rancherUserNotificationController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v3.RancherUserNotification))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateRancherUserNotificationDeepCopyOnChange(client RancherUserNotificationClient, obj *v3.RancherUserNotification, handler func(obj *v3.RancherUserNotification) (*v3.RancherUserNotification, error)) (*v3.RancherUserNotification, error) {
	if obj == nil {
		return obj, nil
	}

	copyObj := obj.DeepCopy()
	newObj, err := handler(copyObj)
	if newObj != nil {
		copyObj = newObj
	}
	if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
		return client.Update(copyObj)
	}

	return copyObj, err
}

func (c *rancherUserNotificationController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controller.RegisterHandler(ctx, name, controller.SharedControllerHandlerFunc(handler))
}

func (c *rancherUserNotificationController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), handler))
}

func (c *rancherUserNotificationController) OnChange(ctx context.Context, name string, sync RancherUserNotificationHandler) {
	c.AddGenericHandler(ctx, name, FromRancherUserNotificationHandlerToHandler(sync))
}

func (c *rancherUserNotificationController) OnRemove(ctx context.Context, name string, sync RancherUserNotificationHandler) {
	c.AddGenericHandler(ctx, name, generic.NewRemoveHandler(name, c.Updater(), FromRancherUserNotificationHandlerToHandler(sync)))
}

func (c *rancherUserNotificationController) Enqueue(name string) {
	c.controller.Enqueue("", name)
}

func (c *rancherUserNotificationController) EnqueueAfter(name string, duration time.Duration) {
	c.controller.EnqueueAfter("", name, duration)
}

func (c *rancherUserNotificationController) Informer() cache.SharedIndexInformer {
	return c.controller.Informer()
}

func (c *rancherUserNotificationController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *rancherUserNotificationController) Cache() RancherUserNotificationCache {
	return &rancherUserNotificationCache{
		indexer:  c.Informer().GetIndexer(),
		resource: c.groupResource,
	}
}

func (c *rancherUserNotificationController) Create(obj *v3.RancherUserNotification) (*v3.RancherUserNotification, error) {
	result := &v3.RancherUserNotification{}
	return result, c.client.Create(context.TODO(), "", obj, result, metav1.CreateOptions{})
}

func (c *rancherUserNotificationController) Update(obj *v3.RancherUserNotification) (*v3.RancherUserNotification, error) {
	result := &v3.RancherUserNotification{}
	return result, c.client.Update(context.TODO(), "", obj, result, metav1.UpdateOptions{})
}

func (c *rancherUserNotificationController) Delete(name string, options *metav1.DeleteOptions) error {
	if options == nil {
		options = &metav1.DeleteOptions{}
	}
	return c.client.Delete(context.TODO(), "", name, *options)
}

func (c *rancherUserNotificationController) Get(name string, options metav1.GetOptions) (*v3.RancherUserNotification, error) {
	result := &v3.RancherUserNotification{}
	return result, c.client.Get(context.TODO(), "", name, result, options)
}

func (c *rancherUserNotificationController) List(opts metav1.ListOptions) (*v3.RancherUserNotificationList, error) {
	result := &v3.RancherUserNotificationList{}
	return result, c.client.List(context.TODO(), "", result, opts)
}

func (c *rancherUserNotificationController) Watch(opts metav1.ListOptions) (watch.Interface, error) {
	return c.client.Watch(context.TODO(), "", opts)
}

func (c *rancherUserNotificationController) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (*v3.RancherUserNotification, error) {
	result := &v3.RancherUserNotification{}
	return result, c.client.Patch(context.TODO(), "", name, pt, data, result, metav1.PatchOptions{}, subresources...)
}

type rancherUserNotificationCache struct {
	indexer  cache.Indexer
	resource schema.GroupResource
}

func (c *rancherUserNotificationCache) Get(name string) (*v3.RancherUserNotification, error) {
	obj, exists, err := c.indexer.GetByKey(name)
	if err != nil {
		return nil, err
	}
	if !exists {
		return nil, errors.NewNotFound(c.resource, name)
	}
	return obj.(*v3.RancherUserNotification), nil
}

func (c *rancherUserNotificationCache) List(selector labels.Selector) (ret []*v3.RancherUserNotification, err error) {

	err = cache.ListAll(c.indexer, selector, func(m interface{}) {
		ret = append(ret, m.(*v3.RancherUserNotification))
	})

	return ret, err
}

func (c *rancherUserNotificationCache) AddIndexer(indexName string, indexer RancherUserNotificationIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v3.RancherUserNotification))
		},
	}))
}

func (c *rancherUserNotificationCache) GetByIndex(indexName, key string) (result []*v3.RancherUserNotification, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	result = make([]*v3.RancherUserNotification, 0, len(objs))
	for _, obj := range objs {
		result = append(result, obj.(*v3.RancherUserNotification))
	}
	return result, nil
}
